umask 022

P=python
V=2.7.14
B=1

# rotate logs
for i in $(seq 9 -1 0); do [ -f package.log.$i ] && mv package.log.$i package.log.$(( i + 1 )); done
[ -f package.log ] && mv package.log package.log.0
exec > >(tee package.log) 2>&1

PREFIX=apps/Python27/
MAINTAINER=JuergenFischer

if ! [ -d "$OSGEO4WREP" ]; then
	echo "OSGEO4WREP not set"
	exit 1
fi

set -e
set -x

exetmpl() {
	local i=$1
	local postinstall=$2
	local b=$(basename $i)
	local t=${PREFIX}Scripts/$b

	echo -e "textreplace -std -t ${t///\\}" >>$postinstall

	perl -pe "s#${PY//\\/\\\\}#\@osgeo4w\@\\\\bin\\\\python.exe#i" $i >$b.tmpl
	chmod a+rx $b.tmpl
}

for a in x86 x86_64; do
	R=$OSGEO4WREP/$a/release
	mkdir -p $R

	case "$a" in
	x86_64)
		PKG=$P-$V.amd64.msi
		;;
	x86)
		PKG=$P-$V.msi
		;;
	esac

	[ -f $PKG ] || curl https://www.python.org/ftp/python/$V/$PKG >$PKG

	if ! [ -d $a ]; then
		msiexec /i $PKG TARGETDIR=$(cygpath -w $PWD/$a)
		[ -d $a ]
	fi

	for p in $P-core $P-help $P-devel $P-test $P-tcltk $P-tools $P-pip setuptools; do
		mkdir -p $R/$P/$p
		cp $a/LICENSE.txt $R/$P/$p/$p-$V-$B.txt
	done

	PIPV=$(sed -ne "s/^Version: //p" $a/Lib/site-packages/pip-*.dist-info/METADATA)
	STV=$(sed -ne "s/^Version: //p" $a/Lib/site-packages/setuptools-*.dist-info/METADATA)

	cat <<EOF >$R/$P/$P-core/setup.hint
sdesc: "Python core interpreter and runtime"
ldesc: "Python core interpreter and runtime"
maintainer: $MAINTAINER
category: Commandline_Utilities
requires: msvcrt2008 sqlite3 
EOF

	PY=$(cygpath -aw $a/python.exe)

	O=
	if [ -f $a/w9xpopen.exe ]; then
		O="$O --xform s,^$a/w9xpopen.exe,bin/w9xpopen.exe, $a/w9xpopen.exe"
	fi

	rm -f pip-postinstall.bat
	exetmpl $a/Scripts/pip.exe pip-postinstall.bat
	exetmpl $a/Scripts/pip2.exe pip-postinstall.bat
	exetmpl $a/Scripts/pip2.7.exe pip-postinstall.bat

	rm -f setuptools-postinstall.bat
	exetmpl $a/Scripts/easy_install-2.7.exe setuptools-postinstall.bat
	exetmpl $a/Scripts/easy_install.exe setuptools-postinstall.bat

	cp $a/Lib/distutils/ccompiler.py ccompiler.py
	patch -p0 <ccompiler.py.diff
	O="$O --exclude $a/Lib/distutils/ccompiler.py --xform s,^ccompiler.py,${PREFIX}Lib/distutils/ccompiler.py, ccompiler.py"

	cat <<EOF >tcltk.lst
$a/Tools/Scripts/pydocgui.pyw
$a/DLLs/_tkinter.pyd
$a/DLLs/tcl85.dll
$a/DLLs/tclpip85.dll
$a/DLLs/tk85.dll
$a/Lib/idlelib
$a/Lib/lib-tk
$a/tcl
EOF

	cat <<EOF >test.lst
$a/DLLs/_ctypes_test.pyd
$a/Lib/bsddb/test
$a/Lib/ctypes/test
$a/Lib/email/test
$a/Lib/lib-tk/test
$a/Lib/sqlite3/test
$a/Lib/lib2to3/tests
$a/Lib/test
$a/Lib/unittest/test
$a/Lib/json/tests
$a/Lib/distutils/tests
$a/Lib/idlelib/idle_test
EOF

	cat <<EOF >pip.lst
./pip.exe.tmpl
./pip2.exe.tmpl
./pip2.7.exe.tmpl
$a/Lib/site-packages/pip
$a/Lib/site-packages/pip-$PIPV.dist-info
EOF

	cat <<EOF >setuptools.lst
./easy_install-2.7.exe.tmpl
./easy_install.exe.tmpl
$a/Lib/site-packages/easy_install.py
$a/Lib/site-packages/setuptools
$a/Lib/site-packages/setuptools-$STV.dist-info
EOF

	cat <<EOF >tools.lst
$a/Tools
EOF

	tar -cjf $R/$P/$P-core/$P-core-$V-$B.tar.bz2 \
		$O \
		--xform "s,^postinstall.bat,etc/postinstall/$P-core.bat," \
		--xform "s,^$a/python.exe,bin/python.exe," \
		--xform "s,^$a/pythonw.exe,bin/pythonw.exe," \
		--xform "s,^$a/python27.dll,bin/python27.dll," \
		--xform "s,^$a/,$PREFIX," \
		--xform "s,^make-bat-for-py.bat,bin/make-bat-for-py.bat," \
		--xform "s,^ini.bat,etc/ini/$P-core.bat," \
		--exclude "*.pyc" \
		--exclude $a/DLLs/sqlite3.dll \
		--exclude-from tcltk.lst \
		--exclude-from test.lst \
		--exclude-from pip.lst \
		--exclude-from setuptools.lst \
		--exclude-from tools.lst \
		make-bat-for-py.bat \
		ini.bat \
		postinstall.bat \
		$a/LICENSE.txt \
		$a/DLLs \
		$a/Lib \
		$a/python.exe \
		$a/pythonw.exe \
		$a/python27.dll \

	cat <<EOF >$R/$P/$P-help/setup.hint
sdesc: "Python documentation in a Windows compiled help file"
ldesc: "Python documentation in a Windows compiled help file"
maintainer: $MAINTAINER
category: Commandline_Utilities
requires: $P-core
external-source: $P-core
EOF

	tar -cjf $R/$P/$P-help/$P-help-$V-$B.tar.bz2 \
		--xform "s,$a/,$PREFIX," \
		$a/Doc/ \
		$a/NEWS.txt \
		$a/README.txt

	cat <<EOF >$R/$P/$P-devel/setup.hint
sdesc: "Python library and header files"
ldesc: "Python library and header files"
maintainer: $MAINTAINER
category: Libs
requires: $P-core
external-source: $P-core
EOF

	tar -cjf $R/$P/$P-devel/$P-devel-$V-$B.tar.bz2 \
		--xform "s,$a/,$PREFIX," \
		$a/include \
		$a/libs

	cat <<EOF >$R/$P/$P-test/setup.hint
sdesc: "Python self tests"
ldesc: "Python self tests"
maintainer: $MAINTAINER
category: Libs
requires: $P-core
external-source: $P-core
EOF

	tar -cjf $R/$P/$P-test/$P-test-$V-$B.tar.bz2 \
		--xform "s,$a/,$PREFIX," \
		--exclude "*.pyc" \
		-T test.lst

	cat <<EOF >$R/$P/$P-tcltk/setup.hint
sdesc: "Python Tkinter and IDLE"
ldesc: "Python Tkinter and IDLE"
maintainer: $MAINTAINER
category: Commandline_Utilities
requires: $P-core
external-source: $P-core
EOF

	tar -cjf $R/$P/$P-tcltk/$P-tcltk-$V-$B.tar.bz2 \
		--xform "s,$a/,$PREFIX," \
		--exclude "*.pyc" \
		--exclude-from "test.lst" \
		-T tcltk.lst

	cat <<EOF >$R/$P/$P-tools/setup.hint
sdesc: "Python Tools"
ldesc: "Python Tools"
maintainer: $MAINTAINER
category: Commandline_Utilities
requires: $P-core
external-source: $P-core
EOF

	tar -cjf $R/$P/$P-tools/$P-tools-$V-$B.tar.bz2 \
		--xform "s,$a/,$PREFIX," \
		--exclude "*.pyc" \
		--exclude-from tcltk.lst \
		$a/Tools

	cat <<EOF >$R/$P/$P-pip/setup.hint
sdesc: "The PyPA recommended tool for installing Python packages."
ldesc: "The PyPA recommended tool for installing Python packages."
maintainer: $MAINTAINER
category: Libs
requires: python-core
EOF

	tar -cjf $R/$P/$P-pip/$P-pip-$PIPV-$B.tar.bz2 \
		--xform "s,^pip-postinstall.bat,etc/postinstall/$P-pip.bat," \
		--xform s,^./pip.exe.tmpl,${PREFIX}Scripts/pip.exe.tmpl, \
		--xform s,^./pip2.exe.tmpl,${PREFIX}Scripts/pip2.exe.tmpl, \
		--xform s,^./pip2.7.exe.tmpl,${PREFIX}Scripts/pip2.7.exe.tmpl, \
		--xform "s,$a/,$PREFIX," \
		--exclude "*.pyc" \
		pip-postinstall.bat \
		-T pip.lst

	cat <<EOF >$R/$P/setuptools/setup.hint
sdesc: "setuptools - Easily download, build, install, upgrade, and uninstall Python packages"
ldesc: "setuptools - Easily download, build, install, upgrade, and uninstall Python packages"
maintainer: $MAINTAINER
category: Libs
requires: shell python-core
EOF
	tar -cjf $R/$P/setuptools/setuptools-$STV-$B.tar.bz2 \
		--xform "s,^setuptools-postinstall.bat,etc/postinstall/setuptools.bat," \
		--xform s,^./easy_install-2.7.exe.tmpl,${PREFIX}Scripts/easy_install-2.7.exe.tmpl, \
		--xform s,^./easy_install.exe.tmpl,${PREFIX}Scripts/easy_install.exe.tmpl, \
		--xform "s,$a/,$PREFIX," \
		--exclude "*.pyc" \
		setuptools-postinstall.bat \
		-T setuptools.lst

	tar -C .. -cjf $R/$P/$P-core/$P-core-$V-$B-src.tar.bz2 \
		osgeo4w/ini.bat \
		osgeo4w/make-bat-for-py.bat \
		osgeo4w/package.sh \
		osgeo4w/postinstall.bat \
		osgeo4w/ccompiler.py.diff
	
	tar -tjf $R/$P/$P-core/$P-core-$V-$B.tar.bz2 | sed -e "s/$/:core/" >/tmp/$P-$a.lst
	tar -tjf $R/$P/$P-help/$P-help-$V-$B.tar.bz2 | sed -e "s/$/:help/" >>/tmp/$P-$a.lst
	tar -tjf $R/$P/$P-devel/$P-devel-$V-$B.tar.bz2 | sed -e "s/$/:devel/" >>/tmp/$P-$a.lst
	tar -tjf $R/$P/$P-test/$P-test-$V-$B.tar.bz2 | sed -e "s/$/:test/" >>/tmp/$P-$a.lst
	tar -tjf $R/$P/$P-tcltk/$P-tcltk-$V-$B.tar.bz2 | sed -e "s/$/:tcltk/" >>/tmp/$P-$a.lst
	tar -tjf $R/$P/$P-tools/$P-tools-$V-$B.tar.bz2 | sed -e "s/$/:tools/" >>/tmp/$P-$a.lst
	tar -tjf $R/$P/$P-pip/$P-pip-$PIPV-$B.tar.bz2 | sed -e "s/$/:pip/" >>/tmp/$P-$a.lst
	tar -tjf $R/$P/setuptools/setuptools-$STV-$B.tar.bz2 | sed -e "s/$/:setuptools/" >>/tmp/$P-$a.lst

	cut -d: -f1 /tmp/$P-$a.lst | sort | uniq -d >/tmp/$P-$a.dupes
	if [ -s /tmp/$P-$a.dupes ]; then
		echo DUPES:
		fgrep -f /tmp/$P-$a.dupes /tmp/$P-$a.lst | sort
		break
	fi
done
